#models.py - The relational database models for an enewspaper.
from __future__ import division
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string
import Image, ImageOps
from urllib import urlretrieve, urlopen, urlcleanup
import operator
import time

from renderarticle import calcPages, create_newsdict, render_article
from image import transcodeImage, saveIll
from index import createIndex
from manifest import makeManifest



upload_path_ill = "/home/hraune/Master_prog/img/"
path_enews_html = "/home/hraune/Master_prog/html/"


#author
class Author(models.Model):
	name = models.CharField(_("name"), max_length = 50)
	slug = models.SlugField(prepopulate_from=('name',))
	
	class Admin:
		pass
	
	def __unicode__(self):
		return self.slug

#illustration
class Illustration(models.Model):
	title = models.CharField(_("title"), max_length = 30)
	slug = models.SlugField(prepopulate_from=('title',))
	caption = models.TextField(_("caption"))
	pathtofile = models.ImageField(_("pathtofile"), upload_to = upload_path_ill)
	enewsversion = models.CharField(_("enews version"), max_length = 60, blank = True)
	author = models.ForeignKey(Author, verbose_name = _("author"))
	uploaded = models.BooleanField(_("check if uploaded from your disk"))
	# eller evt et imagefield
	
	class Admin:
		pass
		
	def __unicode__(self):
		paths = self.pathtofile.split("/")
		return paths[-1]
	
	def save(self):
		self.enewsversion = transcodeImage(self.pathtofile)
		super(Illustration, self).save()
		
	
class Category(models.Model):
	name = models.CharField(_("name"), max_length = 30)
	slug = models.SlugField(prepopulate_from=('name',))
	
	class Admin:
		pass
		
	def __unicode__(self):
		return self.slug

#article
class Article(models.Model):
	author = models.ForeignKey(Author, verbose_name=_("author"))
	category = models.ForeignKey(Category, verbose_name=_("category"))
	title = models.CharField(_("title"), max_length = 50)
	slug = models.SlugField(prepopulate_from=('title',))
	subtitle = models.CharField(_("subtitle"), max_length = 50, blank=True)
	ingress = models.TextField(_("ingress"),)
	body = models.TextField(_("body"),)
	illustration = models.ForeignKey(Illustration, verbose_name=_("illustration"))
	pages = models.IntegerField(_("pages"), blank=True, help_text = _("Do not use this field, it is for stuff under the bonnet"))
	
	class Admin:
		pass
	
	def __unicode__(self):
		return self.slug
		
	def save(self):
		super(Article, self).save()
		render_article(self.slug)  #need to save article before rendering the html versions of the articles
	
#enewspaper
class Enewspaper(models.Model):
	issue = models.CharField(_("issue"), max_length = 20)
	volume = models.CharField(_("volume"), max_length = 20)
	slug = models.SlugField(prepopulate_from=('issue',))
	articles = models.ManyToManyField(Article, verbose_name=_("articles"))
	
	class Admin:
		pass
	
	def __unicode__(self):
		return self.slug

#will populate the database from a given XML.
class XmlToDb(models.Model):
	file = models.FileField(_("File"), upload_to = "/home/hraune/Master_prog/xml_up/")
	
	class Admin:
		pass
		
	def __unicode__(self):
		return "foo"
		

	def save(self):
		from newspaper import Newspaper
		from django.template import defaultfilters
		
		nloader = Newspaper()
		filename = self.get_file_filename()
		articleinfo = nloader.load(filename)
		issue = articleinfo['issue']
		volume = articleinfo['volume']
		articles = articleinfo['article_list']
	
	
		enewspaper = Enewspaper(issue, volume, slug = defaultfilters.slugify(issue))
		enewspaper.save()

		articlelist = []
		categories = []
		for art in articles:
			#print art
			author_text = Author(name = art.author_text, slug = defaultfilters.slugify(art.author_text))
			author_text.save()
			
			author_ill = Author(name = art.author_ill, slug = defaultfilters.slugify(art.author_ill))
			author_ill.save()
			
			cat = Category(name = art.category, slug = defaultfilters.slugify(art.category))
			if art.category not in categories:
				categories.append(cat.name)
				cat.save()
					
			
			ill_filename = pathto_ill.split("/")[-1]
			if pathto_ill.split("/")[0] == "http:":
				saveIll(pathto_ill, upload_path_ill + ill_filename)

			ill = Illustration(	title = pathto_ill.split("/")[-1],
								caption = art.caption,
								slug = defaultfilters.slugify(art.caption[:20]),
								pathtofile = pathto_ill,
								author = author_ill)
			ill.save()
			
			p = calcPages(len(art.body))
			
			a = Article(author = author_text,
						category = cat, 
						title = art.title, 
						slug = defaultfilters.slugify(art.title),
						subtitle = art.subtitle,
						ingress = art.ingress,
						body = art.body,
						illustration = ill,
						pages = p)
			articlelist.append((art.title, cat.name, p))
			a.save()
			
			
			enewspaper.articles.add(a)
			
			slug = defaultfilters.slugify(art.title)
			
		idx = createIndex(categories, articlelist)
		makeManifest((volume +"-"+ issue), idx)
		super(XmlToDb, self).save()
