#onearticle.py

import xml.dom.minidom
from xml.sax.saxutils import escape


class OneArticle:
	"""represents a block of text and metadata created from XML"""

	def __init__(self):
		"""set initial data properties"""
		self.reset()
		
	def reset(self):
		"""function for initializing data properties"""
		#title
		self.title = ""
		self.subtitle = ""
		self.category = ""
		
		#authors
		self.author_text = ""
		self.author_ill = ""
		
		#contents
		self.ingress = ""
		self.body = ""
		
		#management data
		self.size = 0
		self.time = ""
				
	def getXML(self):
		""" returns XML after reassembling from data members that may have changed """
		return ""
		
	def fromXML(self, data):
		"""get article data from an XML document passed as a string."""
		self.reset()
		
		dom = xml.dom.minidom.parseString(data)
		
		self.title = get_attribute(dom, "article", "title")
		self.subtitle = get_attribute(dom, "article", "subtitle")
		self.category = get_attribute(dom, "article", "category")
		
		self.author_text = get_attribute(dom, "author", "text")
		self.author_ill = get_attribute(dom, "author", "ill")
		
		nodelist_i = dom.getElementsByTagName("ingress")
		if nodelist_i:
			assert len(nodelist_i) >= 1
			ingress = nodelist_i[0]
			ingress.normalize()
			if ingress.childNodes:
				self.ingress = ingress.firstChild.data.strip()
		
		nodelist_t = dom.getElementsByTagName("body")
		if nodelist_t:
			assert len(nodelist_t) >= 1
			body = nodelist_t[0]
			body.normalize()
			if body.childNodes:
				self.body = body.firstChild.data.strip()


#helper functions:		

def get_attribute(dom, tagname, attrname):
	"""return the value of a solitary element & attribute if available"""
	nodelist = dom.getElementsByTagName(tagname)
	if nodelist:
		assert len(nodelist) >= 1
		node = nodelist[0]
		return node.getAttribute(attrname).strip()
	else:
		return ""

