#!/usr/bin/python
# coding: utf-8
#simple pdfbuilder with reportlab and platypus


from __future__ import division
from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer, PageBreak ,flowables
from reportlab.lib.units import cm
import reportlab.platypus as platypus
import reportlab.lib
from RSSreader import getArticles, getArticlesAllRSS
from reportlab.lib.pagesizes import A4, A5, A6, landscape, portrait
from reportlab.lib.styles import StyleSheet1, ParagraphStyle
from reportlab.lib.colors import white, black
from reportlab.lib.enums import TA_LEFT, TA_CENTER
import datetime
from BeautifulSoup import BeautifulSoup
from enews.imagemagic import saveIll, transcodeImage
from urllib import urlretrieve, urlopen, urlcleanup
import Image, ImageOps
from time import sleep
import random
from time import strftime
import operator
import pickle
import pyscp
import os.path
import cgi
import tempfile

from reportlab.lib.fonts import addMapping

addMapping('Verdana', 0, 0, 'Verdana')    #normal
addMapping('Verdana', 0, 1, 'Verdana-Italic')     #italic
addMapping('Verdana', 1, 0, 'Verdana-Bold')    #bold
addMapping('Verdana', 1, 1, 'Verdana-BoldItalic')     #italic and bold



#
# The stylesheet used for building the pdf
#
def getStyleSheet():
	stylesheet = StyleSheet1()

	stylesheet.add(ParagraphStyle(name='Normal',
									fontName='Times-Roman', 
									fontSize=10, 
									leading=12))

	stylesheet.add(ParagraphStyle(name='BodyText', 
									parent=stylesheet['Normal'], 
									spaceBefore=6))
									
	stylesheet.add(ParagraphStyle(name='Italic', 
									parent=stylesheet['BodyText'], 
									leading=12,
									spaceBefore=6,
									spaceAfter=3,
									fontName = 'Times-Italic',
									fontSize = 11))

	stylesheet.add(ParagraphStyle(name='Heading1', 
									parent=stylesheet['Normal'], 
									fontName = 'Times-Bold', 
									fontSize=18, 
									leading=22, 
									spaceAfter=6), alias='h1')

	stylesheet.add(ParagraphStyle(name='Title',
									parent=stylesheet['Normal'],
									fontName = 'Times-Bold',
									fontSize=18,
									leading=22,
									alignment=TA_CENTER,
									spaceAfter=6), alias='title')

	stylesheet.add(ParagraphStyle(name='Heading2',
									parent=stylesheet['Normal'],
									fontName = 'Times-Bold',
									fontSize=14,
									leading=18,
									spaceBefore=12,
									spaceAfter=6), alias='h2')

	stylesheet.add(ParagraphStyle(name='Heading3',
									parent=stylesheet['Normal'],
									fontName = 'Times-BoldItalic',
									fontSize=12,
									leading=14,
									spaceBefore=12,
									spaceAfter=6), alias='h3')

	stylesheet.add(ParagraphStyle(name='Bullet',
									parent=stylesheet['Normal'],
									firstLineIndent=0,
									spaceBefore=3), alias='bu')

	stylesheet.add(ParagraphStyle(name='Definition',
									parent=stylesheet['Normal'],
									firstLineIndent=0,
									leftIndent=36,
									bulletIndent=0,
									spaceBefore=6,
									bulletFontName='Times-BoldItalic'), alias='df')

	stylesheet.add(ParagraphStyle(name='Code',
									parent=stylesheet['Normal'],
									fontName='Courier',
									fontSize=8,
									leading=8.8,
									firstLineIndent=0,
									leftIndent=36))
									
	return stylesheet
	
#
#Builds pdf(s) based on an article dict.
#
def makePDF(article_dict, dest_dir="/home/hraune/Master_prog/pdf/"):
	page_width = 351.496063
	page_height = 430.866142
	left_margin = 0.5*cm
	right_margin = 0.5*cm
	top_margin = 0.9*cm
	bottom_margin = 0.5*cm
	
	def myFirstPage(canvas, doc):
		canvas.saveState()
		canvas.setFont('Times-Roman',9)
		canvas.drawString(1.2*cm, page_height - 0.9 * cm, "Under Dusken | Studentavisa i Trondheim")
		canvas.setLineWidth(0.02*cm)
		canvas.line(1.2*cm, page_height - 1*cm, page_width - 1.2*cm, page_height - 1*cm)
		canvas.restoreState()

	def myLaterPages(canvas, doc):
		canvas.saveState()
		canvas.setFont('Times-Roman',9)
		canvas.drawString(1.2*cm, page_height - 0.9 * cm, "Under Dusken | Studentavisa i Trondheim")
		canvas.setLineWidth(0.02*cm)
		canvas.line(1.2*cm, page_height - 1*cm, page_width - 1.2*cm, page_height - 1*cm)
		canvas.restoreState()
	
	art = article_dict
	
	title = art["title"].replace("<h1>", ""). replace("</h1>", "")
	ingress = art["ingress"].replace("<p>", "").replace("</p>", "")
	body = unicode(art["body"], "utf-8")
	author_txt = "Tekst: " + art["author_text"]
	author_ill = "Ill/Foto: " + art["author_ill"]
	
	list_para = []
	no_subheaders = False
	
	if "<h3>" in body:
		tmp = body.split("<h3>")
		for para in tmp:
			p = ""
			if "</h3>" in para:
				h = para.split("</h3>")
				subheader = h[0]
				p = h[1]
				list_para.append((subheader, True))
			else:
				p = para
			p = p.split("<p>")
			for i in p:
				i = i.strip("</p>")
				list_para.append((i, False))
				
	else:
		tmp = body.split("<p>")
		no_subheaders = True
		for para in tmp:
			paras = para.strip("</p>")
			list_para.append((para, False))

	pdf_file_name = dest_dir + title + ".pdf"
	pdf_file_name = pdf_file_name.replace("?", "")

	styles = getStyleSheet()
	h1 = styles["h1"]
	h2 = styles["h2"]
	h3 = styles["h3"]
	normal = styles["Normal"]
	italic = styles["Italic"]
	
	doc = SimpleDocTemplate(pdf_file_name)
	doc.leftMargin = left_margin
	doc.rightMargin = right_margin
	doc.topMargin = top_margin
	doc.bottomMargin = bottom_margin

	# reportlab expects to see XML-compliant
	#  data; need to escape ampersands &c.
	title = cgi.escape(title)
	ingress = cgi.escape(ingress)
		
	### fetch image
	img = ""
	ill_present = False
	ill_save_to_path = "/home/hraune/Master_prog/pdf/ill/"
	img_size = (0,200)	#(height, width): auto width based on aspect ratio if 0
	
	ill_url = ""
	
	if art["ill"] == None:
		ill_present = False
		print "No ill" # str(ill_present)
	else:
		art_ill = art["ill"]
		art_ill_id = int(art_ill.split("/")[-1]) - 1
		ill_url = "http://www.underdusken.no/image/" + str(art_ill_id)# + "/h=200/b="
		ill_present = True
		
	img = None
	
	if ill_present == True:
		print "urlretrieve på:  " + ill_url
		sleep(random.random()*3)
		urlretrieve(ill_url, ill_save_to_path + art["title"] + ".jpeg")
		
		transcoded_img = transcodeImage(ill_save_to_path + art["title"] + ".jpeg", 
												ill_save_to_path, 
												img_size)
		
		if transcoded_img != None:
			aspect = transcoded_img[1]
			aspect = 1.3
			print aspect
			img_h = 4.5  # cm
			img_w = img_h * aspect
		
			print "ttt, " + transcoded_img[0]

			img = flowables.Image(transcoded_img[0], height = img_h *cm , width = img_w *cm)
			ill_present
		else:
			ill_present == False

	ing = (Paragraph(ingress, italic))
	aut_t = (Paragraph(author_txt, normal))
	aut_i = (Paragraph(author_ill, normal))
	
	story = [Paragraph(title, h1)]
	
	if ill_present == True and img != None:
		story.append(flowables.ImageAndFlowables(img,[ing, Spacer(1, .5 * cm), aut_t, aut_i]))	
	else:
		print "uten bilde!!" + str
		story.append(Paragraph(ingress, italic))
		story.append(Spacer(1, .1 * cm))
		story.append(Spacer(1, .1 * cm))
		story.append(Paragraph(author_txt, normal))
		story.append(Paragraph(author_ill, normal))
		story.append(Spacer(1, .1 * cm))
		
	for para in list_para:
		if para[1] == True:
			story.append(Paragraph(para[0], h3))
		else:
			story.append (Paragraph(para[0], normal))
			story.append (Spacer(1, .2 * cm))
	
	
	iliad_dim = (12.4 * cm, 15.2 * cm)
	iliad_dim_pts = (351.496063, 430.866142)
	
	doc.pagesize = portrait(iliad_dim_pts)
	doc.build(story, onFirstPage=myFirstPage, onLaterPages=myLaterPages)
	
	print "Writing to: " + title + ".pdf"
	print
	
	if "?" in article:
		title = title.replace("?", "") + ".pdf"
		
	if "æ" or "ø" or "å" in title:
		title = title.replace("æ", "ae").replace("ø", "oe").replace("å", "aa")
	
	return story

#	
# Builds the pdf-story used for each article
#
def buildStory(article_dict, newspaper="ud", dest_dir="/home/hraune/Master_prog/pdf/"):
	
	art = article_dict
	
	title = art["title"].replace("<h1>", "").replace("</h1>", "")
	category = art["category"]
	ingress = art["ingress"].replace("<p>", ""). replace("</p>", "")
	body = art["body"]
	published = art["published"]
	author_txt = "Tekst: " + art["author_text"]
	author_ill = "Ill/Foto: " + art["author_ill"]
	
	list_para = []
	no_subheaders = False
	
	if "<h3>" in body:
		tmp = body.split("<h3>")
		for para in tmp:
			p = ""
			if "</h3>" in para:
				h = para.split("</h3>")
				subheader = h[0]
				p = h[1]
				list_para.append((subheader, True))
			else:
				p = para
			p = p.split("<p>")
			for i in p:
				i = i.replace("</p>", "")
				list_para.append((i, False))
				
	else:
		tmp = body.split("<p>")
		no_subheaders = True
		for para in tmp:
			paras = para.replace("</p>", "")
			list_para.append((para, False))
	title = cgi.escape(title)
	ingress = cgi.escape(ingress)
	
	### fetch image
	img = None
	ill_present = False
	ill_save_to_path = "/home/hraune/Master_prog/pdf/ill/"
	img_size = (0,200)	#(height, width): auto width based on aspect ratio if 0
	
	ill_url = ""
	
	if art["ill"] == None:
		ill_present = False
		print "No ill" 
	elif newspaper == "ud":
		art_ill = art["ill"]
		art_ill_id = int(art_ill.split("/")[-1]) - 1
		ill_url = "http://www.underdusken.no/image/" + str(art_ill_id)
		ill_present = True
	else:
		ill_url = art["ill"]
	
	
	img_h = 4.5  # cm
	if ill_present == True and art["pickled"] == True:
		ill_path = ill_save_to_path + "enews_" + art["title"] + ".jpeg"
		try:
			print "tries pickle: " + art["title"]
			i = Image.open(ill_path)
			aspect_ratio = i.size[0]/i.size[1]
			img = flowables.Image(ill_save_to_path + "enews_" + art["title"] + ".jpeg", 
									height = img_h *cm, width = img_h * aspect_ratio *cm)
			print "img fetched from pickle: " + art["title"]
		except IOError:
			print "failed pickle"
			art["pickled"] == False
	
	elif ill_present == True and art["pickled"] == False:
		print "urlretrieve på:  " + ill_url
		sleep(random.random()*3)
		urlretrieve(ill_url, ill_save_to_path + art["title"] + ".jpeg")
		transcoded_img = transcodeImage(ill_save_to_path + art["title"] + ".jpeg", 
												ill_save_to_path, 
												img_size)
		
		if transcoded_img != None:
			aspect = transcoded_img[1]
			#aspect = 1.3
			print aspect
			img_w = img_h * aspect
		
			print "ttt, " + transcoded_img[0]
			img = flowables.Image(transcoded_img[0], height = img_h *cm , width = img_w *cm)
			ill_present
		else:
			ill_present == False
			
	styles = getStyleSheet()
	h1 = styles["h1"]
	h2 = styles["h2"]
	h3 = styles["h3"]
	normal = styles["Normal"]
	italic = styles["Italic"]

	ing = (Paragraph(ingress, italic))
	aut_t = (Paragraph(author_txt, normal))
	aut_i = (Paragraph(author_ill, normal))
	
	orig_title = title
	tt = title.lower().replace(" ", "+").replace("\"", "").replace(".", "").replace("&", "og").replace("amp;", "")
	title = '<a name = "'+ tt +'"/> ' + title
	
	story = [Paragraph("<a href='" + art["linktuple"][0] + "'>&lt; Forrige</a> | <a href='#index'>Indeks</a> | <a href='" + art["linktuple"][1] + "'>Neste &gt;</a>", normal)]
	story.append(Paragraph(title, h1))
	if ill_present == True and img != None:
		story.append(flowables.ImageAndFlowables(img,[ing, Spacer(1, .2 * cm), aut_t, aut_i ]))	
	else:
		print "uten bilde!!" + str(orig_title)
		story.append(Paragraph(ingress, italic))
		story.append(Spacer(1, .1 * cm))
		story.append(Paragraph(author_txt, normal))
		story.append(Paragraph(author_ill, normal))
		story.append(Spacer(1, .1 * cm))
		
	for para in list_para:
		if para[1] == True:
			story.append(Paragraph(para[0], h3))
		else:
			story.append (Paragraph(para[0], normal))
			story.append (Spacer(1, .2 * cm))
	
	story.append(PageBreak())		
	return (story, (orig_title, category, published))

#
# Builds the index used in the newspaper
#
def buildIndex(links):
	styles = getStyleSheet()
	h1 = styles["h1"]
	h2 = styles["h2"]
	h3 = styles["h3"]
	normal = styles["Normal"]
	italic = styles["Italic"]
	
	title = '<a name = "' +"index"+ '"/> ' + "Innhold:"
	
	slinks =links
	
	story=[Spacer(1, .6 * cm)]
	story.append(Paragraph(title, h1))
	headers = []  # liste over seksjonsheadere allerede i bruk
	
	print "Slinks: ", len(slinks)
	for link in slinks:
		tt = unicode(link[0].lower().replace(" ", "+").replace("\"", "").replace(".", "").replace("&", "og").replace("amp;", ""), "utf-8")
		#print "tt: " + tt
		print "internal links: " + str(link)
		if link[1] in headers:
			print "inni slinks if"
			link2 = link[2][6:12] + link[2][14:16].replace("-", "/")
			story.append(Paragraph('<a href="#'+ tt +'">' + unicode(link[0], "utf-8") + "("+ link2 +")" + '</a>', normal))
			story.append(Spacer(1, .2 * cm))		
		else:
			print "inni slinks else"
			linktitle = link[1]
			if len(link[1][0])>1:
				linktitle = link[1][0]
			story.append(Paragraph(linktitle, h3))
			headers.append(link[1])
			link2 = link[2][6:12] + link[2][14:16].replace("-", "/")
			story.append(Paragraph('<a href="#'+ tt +'">' + unicode(link[0], "utf-8") + "("+ link2 +")" + '</a>', normal))
			story.append(Spacer(1, .2 * cm))
			
	story.append(PageBreak())	
	
	return story

#
# assembles the entire newspaper
#
def assemblePDFs(rss_url, newspaper="ud", dest_dir="/home/hraune/Master_prog/pdf/"):
	pdf_file_name = dest_dir + newspaper + ".pdf"
	page_width = 351.496063
	page_height = 430.866142
	left_margin = 0.3*cm
	right_margin = 0.3*cm
	top_margin = 0.3*cm
	bottom_margin = 0.1*cm
	
	def myFirstPage(canvas, doc):
		canvas.saveState()
		canvas.setFont('Times-Roman',9)
		canvas.drawString(1.2*cm, page_height - 0.9 * cm, "Under Dusken | Studentavisa i Trondheim")
		canvas.setLineWidth(0.02*cm)
		canvas.line(1*cm, page_height - 1*cm, page_width - 1*cm, page_height - 1*cm)
		canvas.restoreState()
	
	def myLaterPages(canvas, doc):
		canvas.saveState()
		canvas.setFont('Times-Roman',9)
		canvas.restoreState()
	
	doc = SimpleDocTemplate(pdf_file_name)
	doc.leftMargin = left_margin
	doc.rightMargin = right_margin
	doc.topMargin = top_margin
	doc.bottomMargin = bottom_margin
	

	articles = getArticlesAllRSS(rss_url[0], newspaper)

	if articles[1] == False:
		return (False, None) # No new articles
	
	articles = sorted(articles[0], key=operator.itemgetter('category'))

	art_super = []
	temp = []
	category = articles[0]["category"]
	for item in articles:
		if item["category"] == category:
			temp.append(item)
		else:
			art_super.append((category, sorted(temp, key=operator.itemgetter("published"))))
			category = item["category"]
			#print category
			temp = []
			temp.append(item)
	
	art_super.append((category, sorted(temp, key=operator.itemgetter("published"))))
	
	
	articlelist = []
	articles = ["", "", "", "", "", "", "", "", "", ""]
				# [0 "Nyhet", 1 "Reportasje", 2 "Kultur", 3 "Meninger", 
				#  4 "Kommentar", 5 "Kronikk", 6 "Leserbrev", 7 "Editorial", 
				#  8 "Andmeldelse", 9 "UKA"]
	
	for item in art_super:
		print item[0]
		if str(item[0]) == "Nyhet": articles[0] = item[1]
		elif str(item[0]) == "Reportasje": articles[1] = item[1] 
		elif str(item[0]) == "Kultur": articles[2] = item[1]
		elif str(item[0]) == "Meninger": articles[3] = item[1]
		elif str(item[0]) == "Kommentar": articles[4] = item[1]
		elif str(item[0]) == "Kronikker": articles[5] = item[1]
		elif str(item[0]) == "Leserbrev": articles[6] = item[1]
		elif str(item[0]) == "Editorial": articles[7] = item[1]
		elif str(item[0]) == "Anmeldelser": articles[8] = item[1]
		elif str(item[0])[:3] == "UKA": articles[9] = item[1]
		else: print "Passed all categories."
	
	for item in articles:
		articlelist.extend(item)
			
	before = "#index"
	for i in range(len(articlelist)):
		after = ""
		if i < len(articlelist) - 1:
			after = "#" + articlelist[i+1]["title"].lower().replace(" ", "+").replace("\"", "").replace(".", "").replace("&", "og").replace("amp;", "")
		else:
			after = "#index"
		linktuple = (before, after)
		before = "#" + articlelist[i]["title"].lower().replace(" ", "+").replace("\"", "").replace(".", "").replace("&", "og").replace("amp;", "")
		articlelist[i]["linktuple"] = linktuple
	
	stories = []
	links = []
	iter = 0
	for article in articlelist:
		story = buildStory(article)
		links.append(story[1])
		stories.extend(story[0])
	
	storylist = buildIndex(links)
	storylist.extend(stories)
	
	iliad_dim = (12.4 * cm, 15.2 * cm)
	iliad_dim_pts = (351.496063, 430.866142)
	
	doc.pagesize = portrait(iliad_dim_pts)
	doc.build(storylist, onFirstPage=myFirstPage, onLaterPages=myLaterPages)
	
	print "Writing to: " + pdf_file_name + ".pdf"
	print
		
	return (True, pdf_file_name)


#
# Assemble pdfs from other sources than ud
#

def assembleOtherPDFs(rss_url, newspaper, dest_dir="/home/hraune/Master_prog/pdf/"):
	pdf_file_name = dest_dir + newspaper + ".pdf"
	page_width = 351.496063
	page_height = 430.866142
	left_margin = 0.3*cm
	right_margin = 0.3*cm
	top_margin = 0.3*cm
	bottom_margin = 0.1*cm
	
	def myFirstPage(canvas, doc):
		canvas.saveState()
		canvas.setFont('Times-Roman',9)
		if newspaper == "ud":
			canvas.drawString(1.2*cm, page_height - 0.9 * cm, "Under Dusken | Studentavisa i Trondheim")
		elif newspaper == "vg":
			canvas.drawString(1.2*cm, page_height - 0.9 * cm, "Verdens Gang")
		elif newspaper == "db":
			canvas.drawString(1.2*cm, page_height - 0.9 * cm, "Dagbladet")
		elif newspaper == "ad":
			canvas.drawString(1.2*cm, page_height - 0.9 * cm, "Adresseavisen")
		
		canvas.setLineWidth(0.02*cm)
		canvas.line(1*cm, page_height - 1*cm, page_width - 1*cm, page_height - 1*cm)
		canvas.restoreState()
	
	def myLaterPages(canvas, doc):
		canvas.saveState()
		canvas.setFont('Times-Roman',9)
		canvas.restoreState()
	
	doc = SimpleDocTemplate(pdf_file_name)
	doc.leftMargin = left_margin
	doc.rightMargin = right_margin
	doc.topMargin = top_margin
	doc.bottomMargin = bottom_margin
	
	
	new_articles = True
	articles = []
	for url in rss_url:
		art = getArticles(url, newspaper)
		if art[1] == False:
			new_articles = False
		else:
			new_articles = True
		articles.extend(art[0])
		
	if new_articles == False:
		return (False, None) # No new articles
	
	before = "#index"
	for i in range(len(articles)):
		after = ""
		if i < len(articles) - 1:
			after = "#" + articles[i+1]["title"].lower().replace(" ", "+").replace("\"", "").replace(".", "").replace("&", "og").replace("amp;", "")
		else:
			after = "#index"
		linktuple = (before, after)
		before = "#" + articles[i]["title"].lower().replace(" ", "+").replace("\"", "").replace(".", "").replace("&", "og").replace("amp;", "")
		articles[i]["linktuple"] = linktuple
	
	stories = []
	links = []
	iter = 0
	for article in articles:
		story = buildStory(article, newspaper)
		links.append(story[1])
		stories.extend(story[0])
	
	storylist = buildIndex(links)
	storylist.extend(stories)
	
	iliad_dim = (12.4 * cm, 15.2 * cm)
	iliad_dim_pts = (351.496063, 430.866142)
	
	doc.pagesize = portrait(iliad_dim_pts)
	doc.build(storylist, onFirstPage=myFirstPage, onLaterPages=myLaterPages)
	
	print "Writing to: " + pdf_file_name + ".pdf"
	print
	
	return (True, pdf_file_name)


#
#creates a PDF version of the site
#		
def makePDFs(destination="/home/hraune/Master_prog/pdf/"):
	posts = []
	
	def myFirstPage(canvas, doc):
		canvas.line(left_margin, top_margin, right_margin, top_margin)
		canvas.setFont('Times-Italic',12)
		canvas.drawString(left_margin, top_margin + 2, "Homer's Odyssey")
		canvas.line(left_margin, top_margin, right_margin, top_margin)
		
	def myLaterPages(canvas, doc):
		canvas.line(left_margin, top_margin, right_margin, top_margin)
		canvas.setFont('Times-Italic',12)
		canvas.drawString(left_margin, top_margin + 2, "Homer's Odyssey")
		canvas.line(left_margin, top_margin, right_margin, top_margin)
		
	rss_url = "http://www.underdusken.no/nyhet/feed/ud.xml"
	articles = getArticles(rss_url[0])
	
	#let's set up some styles
	cm = reportlab.lib.units.cm
	style = reportlab.lib.styles.getSampleStyleSheet()
	
	#article styles
	headlineStyle = style["Heading2"]
	paraStyle = style["Normal"]
	paraStyle.spaceAfter = 1*cm
	paraStyle.alignment=reportlab.lib.enums.TA_JUSTIFY
	
	#go through each blog post and store its title and content
	for article in articles:
		body = unicode(article["body"], "utf-8")
		title = article["title"]
		#the parts of each post will need to be kept together
		items = []
		headline = platypus.Paragraph(title, headlineStyle)
		items.append(headline)
		for para in body.split("<p>"):
			para = para.strip("</p>")
			#para = paragraph.decode("cp1252")
			para = platypus.Paragraph(para, paraStyle)
			items.append(para)
		item = platypus.KeepTogether(items)
		posts.append(item)
		
	#create the basic page and frames
	document = platypus.BaseDocTemplate(destination + "test.pdf", pagesize=reportlab.lib.pagesizes.A6)
	document.width = 200
	document.leftMargin = 1*cm
	document.rightMargin = 1*cm
	document.topMargin = 1*cm
	document.bottomMargin = 1*cm
	
	frameCount = 1
	frameWidth = document.width/frameCount
	frameHeight = document.height-.02*cm
	frames = []
	#construct a frame for each column
	for frame in range(frameCount):
		leftMargin = document.leftMargin + frame*frameWidth
		column = platypus.Frame(leftMargin, document.bottomMargin, frameWidth, frameHeight)
		frames.append(column)
	template = platypus.PageTemplate(frames=frames)

	document.addPageTemplates(template)
	document.build(posts, onFirstPage=myFirstPage, onLaterPages=myLaterPages)
	
	return 
		
#	
# runs the pdf creator against the given RSS urls.
#
def runner():
	rss_url = "http://www.underdusken.no/nyhet/feed/ud.xml"
	#ud_rss = {"newspaper":"ud" ,"url":["http://ikaros.underdusken.no:8888/feed/ud.xml"]}
	ud_rss = {"newspaper":"ud" ,"url":["http://ikaros.underdusken.no/feed/ud.xml"]}
	db_rss = {"newspaper":"db", "url":[	"http://www.dagbladet.no/rss/nyheter/", 
										"http://www.dagbladet.no/rss/sport/", 
										"http://www.dagbladet.no/rss/kultur/"]}
	vg_rss = {"newspaper":"vg", "url":["http://www1.vg.no/rss/create.php?categories=12,10&keywords=&limit=10",  #nyheter innenriks og utenriks
										"http://www1.vg.no/rss/create.php?categories=20&keywords=&limit=10",   #sport
										"http://www1.vg.no/rss/create.php?categories=25&keywords=&limit=10"]}  #rampelys
	adressa_rss = {"newspaper":"ad", "url":["http://adressa.no/tjenester/rss/feed.jsp?kat=trondheim",
											"http://adressa.no/tjenester/rss/feed.jsp?kat=innenriks",
											"http://adressa.no/tjenester/rss/feed.jsp?kat=utenriks",
											"http://adressa.no/tjenester/rss/feed.jsp?kat=kultur",
											"http://fotball.adressa.no/eliteserien/index.jsp?service=rss"]}
	hw_rss = {"newspaper":"hw", "url":["http://www.hardware.no/feeds/general.xml"]}
	
	newspapers = [ud_rss, vg_rss, db_rss]
	
	for paper in newspapers:
		abbr = ""
		pabbr = ""
		if paper["newspaper"] == "ud":
			pabbr = "UD"
			abbr = ""
		else:
			abbr = paper["newspaper"].upper()
			pabbr = abbr
			
		print abbr
		nextnumber = ""
		pickledir = "/home/hraune/Master_prog/pdf/pickle/" + "contents" + pabbr + ".p"
		contentsfile = "/home/hraune/Master_prog/pdf/" + "contents" + abbr + ".txt"
		if os.path.isfile(pickledir) == True:
			nextnumber = pickle.load(open(pickledir))
			nextnumber = int(nextnumber) + 1
		else:
			print "no nextnumber"
		
		newpaper = ""
		if paper["newspaper"] == "ud":
			newpaper = assemblePDFs(paper["url"], paper["newspaper"])
		else:
			newpaper = assembleOtherPDFs(paper["url"], paper["newspaper"])
	
		if newpaper[0] == True:
			pickle.dump(nextnumber, open(pickledir, "w" ))
			writecontents = open(contentsfile, "w")
			writecontents.write(str(nextnumber))
			writecontents.close()
		
			pyscp.pyscp((contentsfile, newpaper[1]))
			print "Newspaper uploaded"
			
		else:
			print "No new articles... current version is the newest"


if __name__ == "__main__":
	runner()
